/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/stdexcpt.hh>
#include "ascode.hh"
#include "config.h"

using namespace uds;


// misc. implementation

void
DefineFTableHead( ostream& out, const string& name, size_t sz )
{
	string cname = SYM_PREFIX + name;
	
	out<<"\n"
		"\t.section\t.gnu.linkonce.d."<<cname<<",\"aw\",@progbits\n"
		"\t.align "<<sizeof( void* )<<"\n"
		".globl "<<cname<<"\n"
		"\t.type\t"<<cname<<",@object\n"
		"\t.size\t"<<cname<<","<<( sz * sizeof( void* ) )<<endl;
}

void
DefineFTableDef( ostream& out, const string& name )
{
	string tis = name;
	tis.at( 3 ) = 'S';
	tis.erase( tis.rfind( '_' ), string::npos );
	
	out<<SYM_PREFIX<<name<<":\n";
}

void
PrintTIString( ostream& out, const string& name )
{
	string tis = name;
	tis.at( 3 ) = 'S';
	
	string tisv = name.substr( 4 );

	out<<"\n"
		"\t.section\t.gnu.linkonce.r."<<tis<<",\"a\",@progbits\n"
		".globl "<<tis<<"\n"
		"\t.type\t"<<tis<<",@object\n"
		"\t.size\t"<<tis<<","<<( tisv.size() + 1 )<<endl
	   <<tis<<":\n"
		"\t.string\t\""<<tisv<<"\"\n";
}

void
DefineVTable( ostream& out, const string& name, size_t sz )
{
	out<<"\n"
		"\t.section\t.gnu.linkonce.d."<<name<<",\"aw\",@progbits\n"
		"\t.align 8\n"
		".globl "<<name<<"\n"
		"\t.type\t"<<name<<",@object\n"
		"\t.size\t"<<name<<","<<( sz * sizeof( void* ) )<<endl
	   <<name<<":\n";
}

void
PrintTableEntry( ostream& out, const string& v )
{
	out<<"\t.long\t"<<v<<endl;
}

void
EnterTextSegment( ostream& out )
{
	out<<"\n\n.text\n\t.align 4\n";
}

void
DefineFunction( ostream& out, const string& name, bool g )
{
	if ( g )
	{
		// make symbol global
		out<<".globl "<<name<<"\n"
			"\t.type "<<name<<",@function\n";
	}
	
	// define symbol
	out<<name<<":\n";
}

void
AdjustThis( ostream& out, int so, int dt, bool wr )
{
	if ( wr )
	{
		out<<"\tmovl "<<so<<"(%esp),%eax\n"
			"\taddl $"<<dt<<",%eax\n"
			"\tmovl %eax,"<<so<<"(%esp)\n";
	}
	else
		out<<"\taddl $"<<dt<<","<<so<<"(%esp)\n";
}

void
StackDeref( ostream& out, int dt )
{
	out<<"\tmovl "<<dt<<"(%esp),%eax\n";
}

void
WRegDeref( ostream& out, int dt )
{
	out<<"\tmovl "<<dt<<"(%eax),%eax\n";
}

void
WRegJump( ostream& out, int dt )
{
	out<<"\tjmp *"<<dt<<"(%eax)\n";
}

void
Jump( ostream& out, const string& tgt )
{
	out<<"\tjmp "<<tgt<<endl;
}

void
StrConstant( ostream& out, const char* sym, const string& str )
{
	static unsigned L = 0;
	
	out<<"\n.section\t.rodata\n"
		".LC"<<L<<":\n"
		"\t.string\t\""<<str<<"\"\n"
		".data\n"
		"\t.align 4\n"
		"\t.type "<<sym<<",@object\n"
		"\t.size "<<sym<<",4\n"
		".globl "<<sym<<"\n"
		<<sym<<":\n"
		"\t.long .LC"<<L<<"\n";
	
	++L;
}

void
BinConstant( ostream& out, const char* sym, const string& str )
{
	out<<"\n.section\t.rodata\n"
		"\t.type\t"<<sym<<",@object\n"
		"\t.size\t"<<sym<<","<<str.size()<<"\n"
		<<sym<<":\n";
	
	for ( size_t i = 0; i < str.size(); ++i )
		out<<".byte "<<static_cast< int >( str[i] )<<endl;
}

void
EmitNoreturn( ostream& out )
{
	out<<"\n.globl " SYM_PREFIX "noret\n"
		SYM_PREFIX "noret:\n"
		"\tjmp abort\n";
}
