/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_clfilt_hh
#define _frost_clfilt_hh


// includes

#include <uds/check_ptr.hh>
#include <iostream>
#include <queue>
#include <map>
#include <string>

using namespace std;
using namespace uds;


#ifndef yyFlexLexer
#define _CPP_BACKWARD_IOSTREAM_H 1
#include "FlexLexer.h"
#endif


class Class;
class Hierarchy;
class Table;
class VTable;


/** Lexer class
	\author Julian v. Bock */
class ClFilt : public yyFlexLexer
{
public:
	/** \param in input stream
		\param h hierarchy object */
	ClFilt( istream* in, Hierarchy& h );
	
	/** destructor */
	virtual
	~ClFilt();
	
	/** flex lexer method */
	virtual int
	yylex();
	
private:
	typedef queue< Table* > ftab_queue;
	typedef vector< pair< string, int > > vbfo_vec;
	typedef check_ptr< Class > class_ptr;
	typedef check_ptr< VTable > vtable_ptr;

	/// yylex nesting level (used for debugging)
	int			nl;
	/// current class name
	string		cname;
	/// current vtbl table name
	string		tname;
	/// temporary number
	int			cnum;
	/// hierarchy
	Hierarchy&	hier;
	/// current class
	class_ptr	cl;
	/// current virtual table
	vtable_ptr	vtbl;
	/// frost table queue
	ftab_queue	ftab_q;
	/// virtual base class frost table offset queue
	vbfo_vec	vbfo_v;
	/// 'virtual base' flag
	bool		vbase;

	/** processes one frost table and pops it from the queue
		\param cname class name
		\param cnum frost table offset */
	void
	proc_ftable( const string& cname, int cnum );

	/** skips some input
		\param x 'end of line' character */
	void
	skip( char x = '\n' );

	/** \return current input position */
	int
	current_input_pos() const;

	/** sets a new start condition
		\param sc new start condition */
	void
	begin( int sc );

	/** calls yylex() with a new state. the old state is preserved.
		note that this function is not exception safe.
		\param sc new start condition */
	void
	parse( int sc );
};

#endif // _frost_clfilt_hh
