/*
 * Frost - use C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_disp_hh
#define _frost_disp_hh


// includes

#include "dbase.hh"


class TableGrp;


/** represents a dispatcher function
	\author Julian v. Bock */
class Dispatcher : public DispBase
{
public:
	/** \param n dispatcher name
		\param tg frost table group
		\param s fragment section
		\param l fragment level
		\param c offset in fragment
		\param o stack offset of the argument
		\param f frost table offset
		\param g 'id global' flag */
	Dispatcher( const string& n, TableGrp& tg, int s, int l, int c, int o,
				int f, bool g );
	
	/** destructor */
	virtual
	~Dispatcher();
	
	/** prints the dispatcher assembly code
		\param out output stream
		\return output stream */
	virtual ostream&
	EmitCode( ostream& out ) const;

	/** \return if the dispatcher is global */
	virtual bool
	Global() const { return global; }
	
private:
	/// frost table group
	TableGrp&	tgrp;
	/// fragment section
	int			section;
	/// fragment level
	int			level;
	/// offset in fragment
	int			c_offs;
	/// stack offset of the argument
	int			st_offs;
	/// frost table offset
	int			ft_offs;
	/// true if dispatcher is globally visible
	bool		global;
};

#endif // _frost_disp_hh
