/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_fnfilt_hh
#define _frost_fnfilt_hh


// includes

#include <string>
#include <queue>
#include "function.hh"

#ifndef yyFlexLexer
#define _CPP_BACKWARD_IOSTREAM_H 1
#include "FlexLexer.h"
#endif

using namespace std;


class Hierarchy;


/** Lexer class
	\author Julian v. Bock */
class FnFilt : public yyFlexLexer
{
public:
	/** \param in input stream
		\param h hierarchy object */
	FnFilt( istream* in, Hierarchy& h );
	
	/** destructor */
	virtual
	~FnFilt();
	
	/** flex lexer method */
	virtual int
	yylex();
	
private:
	typedef queue< size_t > va_queue_t;

	/// yylex nesting level (used for debugging)
	int				nl;
	/// hierarchy object
	Hierarchy&		hier;
	/// virtual argument number queue
	va_queue_t		va_queue;
	/// current function
	Function::ptr	func;
	/// current arg name
	string			aname;
	/// current arg size
	int				asize;
	/// 'in template arg' flag
	bool			tmplarg;
	/// 'static function' flag
	bool			sfunc;

	/** processes a function argument
		\param ea extra attributes */
	void
	proc_arg( const string& ea = "" );

	/** finishes processing of a function
		\param s 'skip rest of line' flag
		\param d 'drop function' flag */
	void
	finish_func( bool s, bool d );

	/** skips some input
		\param x 'end of line' character */
	void
	skip( char x = '\n' );

	/** \return current input position */
	int
	current_input_pos() const;

	/** sets a new start condition
		\param sc new start condition */
	void
	begin( int sc );

	/** calls yylex() with a new state. the old state is preserved.
		note that this function is not exception safe.
		\param sc new start condition */
	void
	parse( int sc );
};


#endif // _frost_fnfilt_hh
