/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/exit.hh>
#include <uds/init.hh>
#include <uds/sentinel.hh>
#include <uds/sys_util.hh>
#include <exception>
#include <stdlib.h>
#include "progb.hh"
#include "config.h"

using namespace std;


namespace
{

#ifdef FPROFILE
void
ck_gmon_out()
{
	const char* gp = "gmon.out";
	
	if ( uds::FileExist( gp ) )
		rename( gp, uds::TmpFileName( string( "gmon" ) ).c_str() );
}
#endif

}


int
main( int argc, char** arg )
{
	ios::sync_with_stdio( false );

	uds::Init uds_init;
	uds::sentinel().Ignore( "include" );

#ifndef STATIC
	uds::sentinel().IgnoreUnknown();
#endif
	
#ifdef FPROFILE
	atexit( ck_gmon_out );
#endif
	
	try
	{
		Self().CmdLine( vector< string >( arg + 1, arg + argc ) );
		Self().Execute();
		
		return 0;
	}
	catch ( uds::Exit_t& e )
	{
		return e.ReturnValue();
	}
	catch ( uds::bad_cmd_line_opt& e )
	{
		cerr<<e.Info()<<endl;
	}
	catch ( uds::Exception& e )
	{
		cerr<<e.Message();
	}
	catch ( std::exception& e )
	{
		cerr<<"standard exception caught\n"<<e.what()<<"\n\n";
	}
	catch ( ... )
	{
		cerr<<"caught unknown exception\n";
	}
	
	return 2;
}
