/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/io_util.hh>
#include "exception.hh"
#include "sio.hh"

using namespace uds;


namespace
{

void
fr_abort()
{
	if ( !uncaught_exception() )
		throw InvFOFile( "file corrupted" );
}

}


// ListMarker implementation

ListMarker::ListMarker( istream& _in ) :
	in( _in )
{
	if ( in.peek() == ' ' )
		in.get();
	
	if ( in.get() != '(' )
		in.clear( ios::failbit );
}

ListMarker::~ListMarker()
{
	if ( in && in.get() != ')' )
		fr_abort();
}


// misc. implementation

string
ReadQStr( istream& in )
{
	if ( in.peek() == ' ' )
		in.get();
	
	if ( in.get() != '"' )
		fr_abort();

	if ( in.peek() == '"' )
	{
		in.get();
		return "";
	}

	return ReadLine( in, false, true, '"' );
}

bool
ReadBool( istream& in )
{
	if ( in.peek() == ' ' )
		in.get();

	if ( in.get() != '#' )
		fr_abort();

	switch ( in.get() )
	{
	case 't':
		return true;

	case 'f':
		return false;
	}

	fr_abort();
	_uds_abort;
}

void
WalkList( istream& in, const AnyAction& ac, const string& ms )
{
	if ( in.peek() == ' ' )
		in.get();

	if ( in.get() != '(' )
		fr_abort();

	if ( !ms.empty() )
	{
		char* w = new char[ms.size()];

		in.read( w, ms.size() );
		if ( string( w, ms.size() ) != ms )
			fr_abort();

		delete [] w;

		char c = in.get();
		if ( c == ')' )
			return;
		else if ( c != ' ' )
			fr_abort();
	}
	
	if ( in.peek() == ')' )
	{
		in.get();
		return;
	}

	while ( true )
	{
		ac();

		if ( in.get() == ')' )
			return;
	}
}

void
SkipList( istream& in )
{
	while ( true )
	{
		char c = in.get();

		if ( c == '(' )
			SkipList( in );
		else if ( c == ')' )
			return;
		else if ( c == '"' )
			ReadLine( in, false, true, '"' );
	}
}
