/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include "class.hh"
#include "exception.hh"
#include "hierarchy.hh"
#include "progb.hh"
#include "type.hh"


// Type implementation

Type::Type( Hierarchy& h, const string& cn, const string& sr, int sz,
			bool va ) :
	hier( &h ), clname( cn ), srest( sr ), size( sz ), varg( va )
{
}

Type::~Type()
{
}

size_t
Type::Size() const
{
	if ( size < 0 )
	{
		Class* cl = GetClass();
		if ( cl )
			return cl->Size();

		// uh oh, lets hope this is an enum
		_frost_Log( LOG_WARNING, "type " + clname + " unknown, assuming it "
					"is an enum" );

		return sizeof( int );
	}

	return size;
}

Class*
Type::GetClass() const
{
	return hier->QueryClass( clname );
}

bool
Type::PRCType() const
{
	Class* cls = GetClass();
	
	return cls && !cls->BIType()
		&& ( srest == "*" || srest == "&" || srest == " const*"
			 || srest == " const&" );
}

void
Type::SanityCheck() const
{
	if ( varg && !PRCType() )
		throw InvVArg();
}

bool
Type::Compare( const Type& tp, int& m, bool sm ) const
{
	Class* cls = GetClass();
	Class* tp_cls = tp.GetClass();
	
	if ( srest != tp.srest || varg != tp.varg )
		return false;

	if ( cls == tp_cls )
		m = 0;
	else
		if ( !cls || !tp_cls || !cls->Match( *tp_cls, m ) )
			return false;

	// check whether types match exactly unless the 'sm' flag is set
	// or the type in question is a virtual argument

	if ( sm && m != 0 && !varg )
		return false;

	return true;
}


// misc. implementation

ostream&
operator << ( ostream& out, const Type& tp )
{
	return out<<"(\""<<tp.clname<<"\" \""<<tp.srest<<"\" "<<tp.size
			  <<" #"<<( tp.varg ? 't' : 'f' )<<')';
}
