/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_type_hh
#define _frost_type_hh


// includes

#include <iostream>
#include <string>

using namespace std;


class Class;
class Hierarchy;


/** [function argument] type information class.
	\author Julian v. Bock */
class Type
{
public:
	/** \param h hierarchy
		\param cn class name
		\param sr rest of the argument
		\param sz size. if this parameter is < 0 the size is retrieved
		dynamically from the class every time the size is requested.
		\param va 'virtual arg' flag */
	Type( Hierarchy& h, const string& cn, const string& sr, int sz,
		  bool va );

	/** destructor */
	~Type();

	/** \return type size */
	size_t
	Size() const;

	/** \return 'virtual arg' flag */
	bool
	VArg() const { return varg; }

	/** \return true if this argument is a static argument */
	bool
	SArg() const { return size == 0 && GetClass(); }

	/** \return class object */
	Class*
	GetClass() const;

	/** checks whether the type is a pointer or a reference to a class.
		\return true if the type is a pointer or a reference to a class */
	bool
	PRCType() const;

	/** checks whether the type is correct for virtual arguments */
	void
	SanityCheck() const;

	/** compares this type to another one.
		\param tp type to compare
		\param m this parameter is set to 0 if the types match exactly, and
		to the distance if both types are compatible and virtual arguments.
		if the types are not compatible, this parameter is not changed
		and false is returned
		\param sm 'strict match' flag
		\return true if the types are compatible */
	bool
	Compare( const Type& tp, int& m, bool sm ) const;

	/** output stream operator
		\param out output stream
		\param tp type */
	friend ostream&
	operator << ( ostream& out, const Type& tp );

private:
	/// hierarchy
	Hierarchy*	hier;
	/// [possible] class name
	string		clname;
	/// rest of the argument
	string		srest;
	/// type size
	int			size;
	/// 'virtual arg' flag
	bool		varg;
};

#endif // _frost_type_hh
