/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_vtibase_hh
#define _frost_vtibase_hh


// includes

#include <iostream>
#include <string>
#include <vector>

using namespace std;


/** this is a base class for both virtual tables and typeinfo tables.
	\author Julian v. Bock */
class vti_base
{
public:
	/** \param n encoded table name
		\param sz initial table size */
	vti_base( const string& n, size_t sz );

	/** destructor */
	virtual
	~vti_base();

	/** \return vitual table name */
	const string&
	Name() const { return name; }

	/** checks whether all vtable slots are assigned
		\return true if the vtable is complete */
	bool
	Complete() const { return complete && slots.size() == deltas.size(); }

	/** sets the 'complete vtable' flag */
	void
	SetComplete() { complete = true; }

	/** assigns a value to a vtable slot
		\param s vtable slot
		\param v value */
	virtual void
	Assign( size_t s, const string& v ) = 0;

	/** adds raw data to the vtable
		\param v value */
	virtual void
	Fill( int v ) = 0;

	/** print the assembler representation of the virtual table to an output
		stream
		\param out output stream */
	virtual void
	EmitCode( ostream& out ) const;

	/** output stream operator
		\param out output stream
		\param cl class */
	friend ostream&
	operator << ( ostream& out, const vti_base& cl );

protected:
	typedef vector< string > slot_vec;
	typedef vector< int > delta_vec;

	/// vtable slots
	slot_vec	slots;
	/// delta vector
	delta_vec	deltas;

private:
	/// vtable name
	string		name;
	/// 'complete vtable' flag
	bool		complete;
};

#endif // _frost_vtibase_hh
